// pub(crate) mod lowlevel;
pub mod crypto;
pub mod nodes;
pub mod io;
pub mod msg;
pub mod peer;

// pub use nodes::Node;
pub use nodes::NodeService;
pub use msg::MessageService;

pub mod channel_tags {
    pub struct SendPacketTag {}
    pub struct ReceivePacketTag {}
    pub struct MinorTickTag {}
    pub struct MajorTickTag {}
}

#[derive(PartialEq, PartialOrd, Clone)]
pub enum SendOrder {
    Route,
    Sign,
    Encrypt,
    Dispatch,
}
pub type SendPacketChannel = (channel_tags::SendPacketTag, msg::Message, SendOrder);

#[derive(PartialEq, PartialOrd, Clone)]
pub enum ReceiveOrder {
    Decryption,
    Verify,
    Parse,
    Dispatch,
}
pub type ReceivePacketChannel = (channel_tags::ReceivePacketTag, msg::Message, ReceiveOrder);

/// Minor ticks (roughly once per second): for retries, status updates, etc.
pub type MinorTickChannel = (channel_tags::MinorTickTag, ());
/// Major ticks (several times per minute): for retries, cleanup tasks, periodic queries, etc
pub type MajorTickChannel = (channel_tags::MajorTickTag, ());

#[derive(Default)]
pub struct CoreInitService {}

impl crate::service::Service for std::rc::Rc<CoreInitService> {
    fn setup(&self, api: &crate::Fleck) {
        api.create_channel::<MinorTickChannel>();
        api.create_channel::<MajorTickChannel>();
        api.with_service(|io: &io::IOService| {
            io::TimerFdBuilder::default()
                .interval(std::time::Duration::from_secs(1))
                .trigger(|api| api.queue::<MinorTickChannel>(()))
                .build(io);
            io::TimerFdBuilder::default()
                .interval(std::time::Duration::from_secs(15))
                .trigger(|api| api.queue::<MajorTickChannel>(()))
                .build(io);
        });

        api.add_service::<nodes::NodeService>();
    }
}
