//! Peer: someone we can communicate with directly

use std::rc::Rc;

use crate::prelude::*;

pub(crate) enum PeerAddress {
    Stream,
    Udp(std::net::SocketAddr),
    Virtual,
}

pub(crate) struct PeerData {
    pub(crate) io: Rc<dyn super::io::FdHandler>,
    pub(crate) address_info: PeerAddress,
}

impl std::fmt::Debug for PeerData {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.write_str("PeerData {}")
    }
}

#[derive(Clone, Debug)]
pub struct Peer {
    pub(crate) data: Rc<PeerData>,
}

#[derive(Default)]
pub struct PeerService {}

impl Service for std::rc::Rc<PeerService> {
    fn setup(&self, api: &crate::Fleck) {
        api.channel::<fleck_core::SendPacketChannel>()
            .as_ref()
            .sub_opt(fleck_core::SendOrder::Dispatch, self, PeerService::dispatch);
    }
}

impl PeerService {
    fn dispatch(
        &self,
        api: &crate::Fleck,
        msg: super::msg::Message,
    ) -> Option<super::msg::Message> {
        if msg.peer.is_none() {
            log::error!("Message got to dispatch with no peer selected?");
            return None;
        }
        let peer = msg.peer.clone().unwrap();
        peer.data.io.dispatch(api, msg);

        None
    }
}
